`timescale 1ns / 1ps

//// Author:         oaks @cyberchabudai.org
//// Module Name:    matrix_led_ctrl_core
//// Revision:       1.0
//// Tool versions:  QuatusII 11.1
//// Target Devices: EP4CE22F17C6N (DE0-nano)
//// Description:    matrix_led_ctrl core module


module matrix_led_ctrl_core (
	clk, rst_n, 
	ram_radr, ram_rdata,
	led_srck, led_srclr_n, led_srdout, led_col_dout
	);


	//// input and output ////
	input			clk;
	input			rst_n;

	output	[3:0]	ram_radr;
	input	[31:0]	ram_rdata;

	output			led_srck;
	output			led_srclr_n;
	output			led_srdout;
	output	[15:0]	led_col_dout;


	//// reg and wire ////
	reg		[5:0]	state;
	reg		[5:0]	state_new;

	reg		[32:0]	sftdat_reg;
	reg		[2:0]	sftclkcnt_reg;
	reg				sftck_reg;
	reg		[7:0]	sftclrcnt_reg;
	reg				sftclr_reg;

	reg		[3:0]	colcnt_reg;
	reg		[9:0]	dispclk_reg;
	reg		[3:0]	brcnt_reg;
	reg		[15:0]	colout_pre_reg;
	reg		[15:0]	colout_reg;

	wire			loadramend_net;
	wire			sftdataend_net;

	wire			brcnt_net;
	wire			brcntend_net;
	wire			colend_net;



	//// parameters ////
	parameter	LEDBRVAL	= 4'hF;			// LED brightness (duty cycle of sink driver)

	// parameter of state machime
	parameter	LOADMEM		= 3'b001;		// load ram data to internal shift register, external shift register clear
	parameter	SFTOUT		= 3'b010;		// output clock and data to external shift register
	parameter	LED_ON		= 3'b100;		// turn on sink driver during time of LEDBRVAL setting



	//// state machime of Matrix LED control ////

	always @(*) begin
		case (state)
			LOADMEM:	if (loadramend_net)
							state_new <= SFTOUT;
						else
							state_new <= state;

			SFTOUT:		if (sftdataend_net)
							state_new <= LED_ON;
						else
							state_new <= state;

			LED_ON:		if (brcntend_net)
							state_new <= LOADMEM;
						else
							state_new <= state;

			default:	state_new <= LOADMEM;
		endcase
	end

	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			state <= LOADMEM;
		else
			state <= state_new;
	end	



	//// LED Row line Drive                                ///
	//// generate data, clock and clear signal for 74xx164 ///

	// counter to update "sftck_reg", "sftdat_reg"
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			sftclkcnt_reg <= 3'd0;
		else if (state == SFTOUT && sftclkcnt_reg <= 3'd5)
			sftclkcnt_reg <= sftclkcnt_reg + 3'd1;
		else
			sftclkcnt_reg <= 3'd0;
	end


	// ext shift register clock (10MHz)
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			sftck_reg <= 1'b0;
		else if (state == SFTOUT && sftclkcnt_reg <= 3'd1)
			sftck_reg <= 1'b1;
		else
			sftck_reg <= 1'b0;
	end

	assign led_srck = sftck_reg;


	// ext shift register data
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			sftdat_reg <= 33'd0;
		else if (state == LOADMEM)
			sftdat_reg <= {1'b1, ram_rdata};
		else if ((sftclkcnt_reg == 3'd3) & ~sftdataend_net)
			sftdat_reg <= {1'b0, sftdat_reg[32:1]};
		else
			sftdat_reg <= sftdat_reg;
	end

	assign sftdataend_net = (sftdat_reg == 33'h000000001);
	assign led_srdout = sftdat_reg[0];


	// ext shift register clear
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			sftclrcnt_reg <= 8'h00;
		else if (state == LOADMEM)
			sftclrcnt_reg <= sftclrcnt_reg + 8'h01;
		else
			sftclrcnt_reg <= 8'h00;
	end

	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			sftclr_reg <= 1'b1;
		else if (sftclrcnt_reg >= 8'he0 && sftclrcnt_reg <= 8'hef)
			sftclr_reg <= 1'b0;
		else
			sftclr_reg <= 1'b1;
	end

	assign led_srclr_n = sftclr_reg;
	assign loadramend_net = (sftclrcnt_reg == 8'hff);




	//// LED Column line (sink) Drive ////

	// column-line counter
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			colcnt_reg <= 4'h0;
		else if (brcntend_net)
			colcnt_reg <= colcnt_reg + 4'h1;
		else
			colcnt_reg <= colcnt_reg;
	end

	assign ram_radr = colcnt_reg;


	// generate base clk for display : 50MHz/1024
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			dispclk_reg <= 10'd0;
		else if (state == LED_ON)
			dispclk_reg <= dispclk_reg + 10'd1;
		else
			dispclk_reg <= 10'd0;
	end


	// LED brightness control
	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			brcnt_reg <= 4'h0;
		else if (state == LED_ON && dispclk_reg == 10'h3FF)
			brcnt_reg <= brcnt_reg + 4'h1;
		else
			brcnt_reg <= brcnt_reg;
	end

	assign brcnt_net = (brcnt_reg >= 4'h1 && brcnt_reg <= LEDBRVAL);
	assign brcntend_net = (brcnt_reg == 4'hF && dispclk_reg == 10'h3FF);


	// output sink driver (16bit parallel)
	always @(*) begin
		case (colcnt_reg)
			4'h0 : colout_pre_reg <= 16'b0000000000000001 & {16{brcnt_net}};
			4'h1 : colout_pre_reg <= 16'b0000000000000010 & {16{brcnt_net}};
			4'h2 : colout_pre_reg <= 16'b0000000000000100 & {16{brcnt_net}};
			4'h3 : colout_pre_reg <= 16'b0000000000001000 & {16{brcnt_net}};
			4'h4 : colout_pre_reg <= 16'b0000000000010000 & {16{brcnt_net}};
			4'h5 : colout_pre_reg <= 16'b0000000000100000 & {16{brcnt_net}};
			4'h6 : colout_pre_reg <= 16'b0000000001000000 & {16{brcnt_net}};
			4'h7 : colout_pre_reg <= 16'b0000000010000000 & {16{brcnt_net}};
			4'h8 : colout_pre_reg <= 16'b0000000100000000 & {16{brcnt_net}};
			4'h9 : colout_pre_reg <= 16'b0000001000000000 & {16{brcnt_net}};
			4'ha : colout_pre_reg <= 16'b0000010000000000 & {16{brcnt_net}};
			4'hb : colout_pre_reg <= 16'b0000100000000000 & {16{brcnt_net}};
			4'hc : colout_pre_reg <= 16'b0001000000000000 & {16{brcnt_net}};
			4'hd : colout_pre_reg <= 16'b0010000000000000 & {16{brcnt_net}};
			4'he : colout_pre_reg <= 16'b0100000000000000 & {16{brcnt_net}};
			4'hf : colout_pre_reg <= 16'b1000000000000000 & {16{brcnt_net}};
			default : colout_pre_reg <= 16'b0000000000000000;
		endcase
	end

	always @(posedge clk or negedge rst_n) begin
		if (!rst_n)
			colout_reg <= 1'b0;
		else
			colout_reg <= colout_pre_reg;
	end

	assign led_col_dout = colout_reg;



endmodule
